/*
 * HMC5883.c
 * 
 * Copyright 2013 Stefan Mavrodiev <support@olimex.com>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA.
 */
#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <math.h>
#include "../inc/HMC5883.h"
#include "i2c.h"

#define M_PI 3.1415926535897932384626338327
int debug = 0;
float declination = 0;
float resolution = 0.92;    // The default value @ 1090 GAIN @ 1.3Ga

struct _registers_t regs[] = {
    {"Configuration Register A", 0x00, 0x10, 0x10},
    {"Configuration Register B", 0x01, 0x00, 0x00},
    {"Mode register", 0x02, 0x00, 0x00},
    {"Data Output X MSB Register", 0x03, 0x00, 0x00},
    {"Data Output X LSB Register", 0x04, 0x00, 0x00},
    {"Data Output Z MSB Register", 0x05, 0x00, 0x00},
    {"Data Output Z LSB Register", 0x06, 0x00, 0x00},
    {"Data Output Y MSB Register", 0x07, 0x00, 0x00},
    {"Data Output Y LSB Register", 0x08, 0x00, 0x00},
    {"Status Register", 0x09, 0x00, 0x00},
    {"Identification Register A", 0x0A, 0x48, 0x48},
    {"Identification Register B", 0x0B, 0x34, 0x34},
    {"Identification Register C", 0x0C, 0x33, 0x33}
    
};

void list_output_rate(void){
    printf("\nTypical Data Output Rate (Hz)\n");
    printf("--------------------------------\n");
    printf("0.75\n1.5\n3\n7.5\n15\n30\n75\n");
}
void list_measurement_mode(void){
    printf("\nMeasurement Mode\n");
    printf("--------------------------------\n");
    printf("normal\npositive\nnegative\n");
}
void list_gain(void){
    printf("\nGain (LSb/Gauss)\n");
    printf("--------------------------------\n");
    printf("1370\n1090\n820\n660\n440\n390\n330\n230\n");
}
void list_operating_mode(void){
    printf("\nOperating mode\n");
    printf("--------------------------------\n");
    printf("continuous\nsingle\nidle\n");
}
int InitializeModule(int *fd, char *i2c, unsigned char address){
    unsigned char in_buffer[3];
    unsigned char out_buffer[5];
    
    
    
    if(debug)
        printf("Opening %s @ 0x%02X...\n", i2c, address);
    if(I2C_Open(fd, i2c, address) < 0){
        printf("Failed to open i2c-device!\n");
        exit(EXIT_FAILURE);
    }
    /* Checking ID... */
    if(debug)
        printf("Checking id...");
    out_buffer[0] = 0x0A;
    I2C_Send(fd, out_buffer, 1);
    I2C_Read(fd, in_buffer, 3);
    if(in_buffer[0] != 0x48 || in_buffer[1] != 0x34 || in_buffer[2] != 0x33){
        printf("Device id doesn't match!\n");
        exit(EXIT_FAILURE);
    }
    if(debug)
        printf("OK\n");
    
    /* Sending configuration */
    out_buffer[0] = 0x00;
    out_buffer[1] = regs[CONF_REG_A].current_value;
    out_buffer[2] = regs[CONF_REG_B].current_value;
    out_buffer[3] = regs[MOD_REG].current_value;
    if(debug)
        printf("Sending configuration bytes...");
    if(I2C_Send(fd, out_buffer, 4) < 0){
        printf("Failed to send data!\n");
        exit(EXIT_FAILURE);
    }
    if(debug)
        printf("OK\n");
    
  
    
    return(EXIT_SUCCESS);
}

float GetHeading(int *fd){
    int x = 0, y = 0, z = 0;
    float rads = 0;
    unsigned char out_buffer[1];
    unsigned char in_buffer[6];
    
    /*Wait until data is ready*/

    if(debug)
        printf("Waiting for data to be ready...\n");

    out_buffer[0] = regs[STATUS_REG].address;
    while(1){
        I2C_Send(fd, out_buffer, 1);
        I2C_Read(fd, in_buffer, 1);
        if(in_buffer[0] & 0x01)
            break;
    }
    /* read data output */    
    out_buffer[0] = regs[OUTPUT_X].address;
    if(I2C_Send(fd, out_buffer, 1) < 0){
        printf("Failed to send data!\n");
        exit(EXIT_FAILURE);
    }
    if(I2C_Read(fd, in_buffer, 6) < 0) {
        printf("Failed to raed data!\n");
        exit(EXIT_FAILURE);
    }
    x = (short)(in_buffer[0] << 8) | in_buffer[1];
    y = (short)(in_buffer[4] << 8) | in_buffer[5];
    z = (short)(in_buffer[2] << 8) | in_buffer[3];
    if(debug){
        printf("RAW X: %d\t", x);
        printf("RAW Y: %d\t", y);
        printf("RAW Z: %d\t", z);
    }
    
    rads = atan2(y, x);
    if(rads < 0)
      rads += 2 * M_PI;
    return rads;    
}
